import Globals
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.Archetypes.public import RichWidget
from Products.CMFCore import CMFCorePermissions

from Products.CopyrightedFile.config import INPUT_TYPE, BARCHART_COLORS
from Products.CopyrightedFile.Permissions import *

from Products.ATContentTypes.content.document import ATDocument
from Products.ATContentTypes.content.document import finalizeATCTSchema

from Products.Archetypes.Storage import *

schema = ATDocument.schema.copy()
schema = schema + Schema((

    LinesField('answerOptions',
        searchable=0,
        required=0,
        default=("I Accept"),
        widget=LinesWidget(
            label="Answer options",
            label_msgid="label_answer_options",
            description="""Enter the options you want to be available to the user here.
                           This only applies to the selection box, multiple selection box,
                           checkbox and radio button input types (selected below).
                           The text input options allow the user to type the input
                           rather than selecting from options. Press enter to seperate
                           the options.""",
            description_msgid="help_answer_options"),
        ),

    StringField('inputType',
        searchable=0,
        required=0,
        vocabulary=INPUT_TYPE,
        default=('checkbox'),
        widget=SelectionWidget(
            label="Input Type",
            label_msgid="label_input_type",
            description="Please select what type of input you would like to use for this question.",
            description_msgid="help_input_type"),
        ),

    BooleanField('required',
        searchable=0,
        required=0,
        default=1,
        widget=BooleanWidget(
            label="Required",
            label_msgid="label_required",
            description="Select if this question is required, meaning participant must give a response.",
            description_msgid="help_required"),
        ),

    StringField('answers',
        searchable=0,
        required=0,
        default={},
        widget=StringWidget(visible=0,),
        ),
     ))

finalizeATCTSchema(schema)

class Copyright(ATDocument):
    """The copyright of a copyrighted file"""
    archetype_name = portal_type = "Copyright"
    meta_type = "Copyright"
    content_icon = "document_icon.gif"

    schema = schema
    actions = ATDocument.actions
    #immediate_view = "base_edit"

    global_allow = 0
    filter_content_types = 1
    allowed_content_types = ()
    include_default_actions = 1
    _at_rename_after_creation = True

    security = ClassSecurityInfo()

    # Set security for the object itslef
    #security.declareObjectProtected('CopyrightedFile: View CopyrightedFile')

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'resetForUser')
    def resetForUser(self, userid):
        """Remove answer for a single user"""
        answers = self.getAnswers()
        if answers.has_key(userid):
            del answers[userid]
        self.setAnswers(answers)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'addAnswer')
    def addAnswer(self, userid, value, comments=""):
        """Add an answer and optional comments for a user."""
        answers = self.getAnswers()

        answers[userid] = {'value':value,
                           'comments':comments}
        self.setAnswers(answers)

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getAnswerFor')
    def getAnswerFor(self, userid):
        """Get a specific user's answer"""

        answer = self.answers.get(userid, {}).get('value', None)
        return answer

Globals.InitializeClass(Copyright)
registerType(Copyright)
