import string
import Globals
from DateTime import DateTime

from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *

# needed for getAnonymousID
from Products.CMFCore.utils import getToolByName

from Products.CopyrightedFile.config import NOTIFICATION_METHOD
from Products.CopyrightedFile.interfaces import ICopyrightedFile
from Products.CopyrightedFile.Permissions import *

from AccessControl.SecurityManagement import newSecurityManager, noSecurityManager

from Products.ATContentTypes.content.folder import ATFolder
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

HeaderSchema = ATFolder.schema.copy()
del HeaderSchema['id']

schema = HeaderSchema + Schema((

    StringField('title',
        required=1,
        searchable=1,
        default='',
        accessor='getTitle',
        widget=StringWidget(
            label="Title",
            label_msgid="label_title",
            description = "Enter a title.",
            description_msgid="help_title"),
    ),

    TextField('thankYouMessage',
        required=0,
        searchable=0,
        default="""Thank you for downloading LOINC.""",
        widget=TextAreaWidget(
            label="'Thank you' message text",
            label_msgid="label_thanku",
            description="""This is the message that will be displayed to the
                           user when they finish viewing/downloading the file.""",
            description_msgid="help_thanku"),
        ),

    TextField('savedMessage',
        required=0,
        searchable=0,
        default="""You have saved the copyright viewing.
Don't forget to come back and finish it.""",
        widget=TextAreaWidget(
            label="'Saved' message test",
            label_msgid="label_saved_text",
            description="""This is the message that will be displayed to the user
                           when they save the copyright viewing, but don't complete the viewing/downloading.""",
            description_msgid="label_saved_text"),
        ),

    StringField('exitUrl',
        required=0,
        searchable=0,
        widget=StringWidget(
            label="Exit URL",
            label_msgid="label_exit_url",
            description="""This is the URL that the user will be directed to on completing the viewing/downloading of the file.
                           Use "http://site.to.go.to/page" or "route/to/page" for this portal""",
            description_msgid="help_exit_url"),
        ),

    BooleanField('allowAnonymous',
        searchable=0,
        required=0,
        widget=BooleanWidget(
            label="Allow Anonymous",
            label_msgid="label_allow_anonymous",),
        ),

    StringField('fileNotificationEmail',
        required=0,
        searchable=0,
        widget=StringWidget(
            label="File Notification Email Address",
            label_msgid="label_file_notification_email",
            description="Enter an email address to receive notifications of file downloads.",
            description_msgid="help_file_notification_email"),
        ),

    StringField('fileNotificationMethod',
        required=0,
        searchable=0,
        vocabulary=NOTIFICATION_METHOD,
        widget=SelectionWidget(
            label="File Notification Method",
            label_msgid="label_file_notification_method",
            description="Select a method to receive notification emails.",
            description_msgid="help_file_notification_method"),
        ),

    StringField('completedFor',
        searchable=0,
        required=0,
        default=[],
        widget=StringWidget(visible=0,),
        ),
        
    IntegerField('viewCount',
        searchable=0,
        required=0,
        default=0,
        widget=IntegerWidget(visible=0,),
        ),

    ))

finalizeATCTSchema(schema)

Actions = (
    {
    'id' : 'manage',
    'name' : 'Manage CopyrightedFile',
    'action' : 'string:${object_url}/CopyrightedFile_view_respondants',
    'permissions' : (EDIT_CONTENTS_PERMISSION, )
    },

)

class CopyrightedFile(ATFolder):
    """You can add contents to CopyrightedFile Folder"""

    portal_type = meta_type = 'CopyrightedFile'
    archetype_name = 'CopyrightedFile'
    content_icon = "icon_CopyrightedFile_small.gif"
    typeDescription= 'A file that has an associated copyright'
    typeDescMsgId  = 'CopyrightedFile_description_edit'
    allowed_content_types = (['Copyright', 'FileWithCopyright', 'SubCopyrightedFile'])

    # Set up our views - these are available from the 'display' menu
    default_view = 'CopyrightedFile_view_base'
    immediate_view = 'CopyrightedFile_view_base'
    suppl_views = ('CopyrightedFile_view_base','CopyrightedFile_view_alternate')

    _at_rename_after_creation = True

    schema = schema
    actions = ATFolder.actions + Actions
    global_allow = 1
    filter_content_types = 1
    include_default_actions = 1

    __implements__ = OrderedBaseFolder.__implements__ + \
                     ATFolder.__implements__
     #                (ICopyrightedFile)

    security = ClassSecurityInfo()
    
    # Set security for the object itslef

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'Title')
    def Title(self, **kw):
        # Removed rename with status
        """Don't override DC Title to include its status: open or closed"""
        title = self.getField('title').get(self)
        return title

#    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getTitle')
#    def getTitle(self):
#        """return the title, or raw title if in edit form"""
#        # check if in base_edit
#        path_info = self.REQUEST.PATH_INFO
#        if path_info.split('/')[-1:] == ['atct_edit']:
#            return self.getField('title').get(self)
#        return self.Title()

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getAbstract')
    def getAbstract(self, **kw):
        return self.Description()

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'setAbstract')
    def setAbstract(self, val, **kw):
        self.setDescription(val)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'isMultipage')
    def isMultipage(self):
        """Return true if there is more than one page in the copyrighted file listing"""
        if self.getFolderListingFolderContents(contentFilter={'portal_type':'SubCopyrightedFile',}):
            return True


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getCopyrights')
    def getCopyrights(self):
        """Return the copyrights for this copyrighted file"""

        contents = self.getFolderListingFolderContents(contentFilter={'portal_type':'Copyright',})
        return contents


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getCopyrightedFiles')
    def getCopyrightedFiles(self):
        """Return the copyrighted file"""


        contents = self.getFolderListingFolderContents(contentFilter={'portal_type':'FileWithCopyright',})
        exit = contents[0].absolute_url() + '/' + contents[0].getId() + contents[0].getSecretView()
        self.REQUEST.RESPONSE.setHeader('Content-Type', contents[0].getContentType())
        self.REQUEST.RESPONSE.setHeader('Content-Disposition', 'attachment;filename=' + contents[0].getId())
        return contents[0].getFile()

        #
        #return self.REQUEST.RESPONSE.redirect(exit)
        
    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getCopyrightedFiles2')
    def getCopyrightedFiles2(self):
        """Returns all the copyrighted files in a folder"""

        contents = self.getFolderListingFolderContents(contentFilter={'portal_type':'FileWithCopyright',})
        return contents

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'gotoCopyrightedFile')
    def gotoCopyrightedFile(self):
        """Return the copyrighted file"""
        contents = self.getFolderListingFolderContents(contentFilter={'portal_type':'FileWithCopyright',})
        exit = contents[0].absolute_url() + '/' + contents[0].getId() + '/view'
        #
        
        self.setCompletedForUser()
        return contents[0].view()

        #
        #return self.REQUEST.RESPONSE.redirect(exit)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getAllContents')
    def getAllContents(self):
        """Return all the contents in the copyrighted file"""
        portal_catalog = getToolByName(self, 'portal_catalog')
        contents = []
        path = string.join(self.getPhysicalPath(), '/')
        results = portal_catalog.searchResults('portal_type' in ('Copyright', 'FileWithCopyright'),
                                               path = path)
        for result in results:
            questions.append(result.getObject())
        return contents

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getNextPage')
    def getNextPage(self):
        """Return the next page of the copyrighted file"""

        pages = self.getFolderListingFolderContents(contentFilter={'portal_type':'SubCopyrightedFile',})
        if pages:
            return pages[0]()
        # no next page, so copyrighted file finished
        self.setCompletedForUser()
        return self.exitCopyrightedFile()


    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'exitCopyrightedFile')
    def exitCopyrightedFile(self):
        """Return the defined exit url"""
        exit_url = self.getField('exitUrl').get(self)
        if exit_url[:7] != 'http://':
            exit_url = self.portal_url() + '/' + exit_url + '?portal_status_message=' + self.getThankYouMessage()
        return self.REQUEST.RESPONSE.redirect(exit_url)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'setCompletedForUser')
    def setCompletedForUser(self):
        """Set completed for a user"""
        userid = self.getCopyrightedFileId()
        completed = self.getCompletedFor()
        completed.append(userid)
        self.setCompletedFor(completed)
        if self.getFileNotificationMethod() == 'each_submission':
            self.send_email()

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'checkCompletedFor')
    def checkCompletedFor(self, user_id):
        """Check whether a user has viewed/downloaded the copyrighted file"""
        completed = self.getCompletedFor()
        if user_id in completed:
            return True
        return False

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'canViewContents')
    def canViewCotents(self):
        """Check if user can view copyrighted file"""
        # copied from skins, does not work yet

        # this this is better handled within Plone ...

        if context.portal_membership.isAnonymousUser() and not context.getAllowAnonymous():
            return 0
        return not context.checkCompletedFor(user_id)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getCopyrightedFileId')
    def getCopyrightedFileId(self):
        """Return the userid for the CopyrightedFile"""
        mt = getToolByName(self, 'portal_membership')
        if mt.isAnonymousUser(): # the user has not logged in
            if self.getAllowAnonymous():
                  current_user = self.getAnonymousId()
                  return current_user
            else:
                 pass
        else:
            member = mt.getAuthenticatedMember()
            current_user = member.getUserName()
            return current_user
        return mt.getAuthenticatedMember().getUserName()
        
    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getMembInst')
    def getMembInst(self, userid, ip_addr):
        """Return the institution for a member"""
        mt = getToolByName(self, 'portal_membership')
        if mt.isAnonymousUser():
             return ip_addr
        else:
             member = mt.getAuthenticatedMember()
             institution = member.getProperty('organization','')
             return institution

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getAnonymousId')
    def getAnonymousId(self):
    # returns the id to use for an anonymous user
        portal_membership = getToolByName(self, 'portal_membership')
        if portal_membership.isAnonymousUser() and self.getAllowAnonymous():
            if not hasattr(self, 'CopyrightedFile_id_no'):
                self.CopyrightedFile_id_no = 0
            self.CopyrightedFile_id_no += 1
            return 'anonymous' + str(self.CopyrightedFile_id_no)
        else:
            # XXX raise unathorized error
            pass
        return portal_membership.getAuthenticatedMember().getUserName()

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getRespondents')
    def getRespondents(self):
        """Return a list of respondents"""
        respondents = self.getCompletedFor()
        users = {}
        for user in respondents:
            users[user] = 1
        return users.keys()

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getAnswersByUser')
    def getAnswersByUser(self, userid):
        """Return a set of answers by user id"""
        contents = self.getContents()
        answers = {}
        for content in contentss:
            answer = content.getAnswerFor(userid)
            answer[content.getId()] = answer
        return answers

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getContentsCount')
    def getContentCount(self):
        """Return a count of contents viewed"""
        return len(self.questions)

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'resetForUser')
    def resetForUser(self, userid):
        """Remove answer for a single user"""
        completed = self.getCompletedFor()
        if userid in completed:
            completed.remove(userid)
        self.setCompletedFor(completed)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'send_email')
    def send_email(self):
        """ Send email to nominated address """
        properties = self.portal_properties.site_properties
        mTo = self.getFileNotificationEmail()
        mFrom = properties.email_from_address
        mSubj = '[%s] New CopyrightedFile  viewed' % self.Title()
        message = []
        message.append('Greetings,')
        message.append('A user has viewed and downloaded a Copyrighted file.')
        mMsg = '\n\n'.join(message)
        try:
            self.MailHost.send(mMsg, mTo, mFrom, mSubj)
        except:
            # XXX too many things can go wrong
            pass
Globals.InitializeClass(CopyrightedFile)
registerType(CopyrightedFile)
