import Globals
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.Archetypes.public import RichWidget
from Products.CMFCore import CMFCorePermissions
from Products.ATContentTypes.configuration import zconf

from Products.CopyrightedFile.config import INPUT_TYPE, COMMENT_TYPE, BARCHART_COLORS
from Products.CopyrightedFile.Permissions import *
from Products.CopyrightedFile.session_validator import *

from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.Archetypes.SQLStorage import MySQLSQLStorage
from Products.Archetypes.SQLMethod import *
from Products.Archetypes.SQLStorageConfig import *
from Products.Archetypes.Storage import *
from Products.ATContentTypes.config import ICONMAP
from urllib import quote

from Products.validation.validators.SupplValidators import MaxSizeValidator

from Products.validation.config import validation
from Products.validation import V_REQUIRED
validation.register(MaxSizeValidator('checkFileMaxSize',
                                     maxsize=zconf.ATFile.max_file_size))

schema = ATCTContent.schema.copy()
schema = schema + Schema((

    FileField('CopyrightedFile',
        required=1,
        searchable=False,
        primary=True,
        storage = AnnotationStorage(migrate=True),
        validators = (('isNonEmptyFile', V_REQUIRED),
                             ('checkFileMaxSize', V_REQUIRED)),
        widget = FileWidget(
              #description = "Select the file to be added by clicking the 'Browse' button.",
              #description_msgid = "help_file",
              description = "",
              label= "Copyrighted File",
              label_msgid = "label_file",
              i18n_domain = "plone",
              show_content_type = False,
              assocMimetypes = ('application/*', 'audio/*', 'video/*', ),
              assocFileExt   = (),)
        ),

    TextField('Body',
              required=False,
              searchable=False,
              primary=False,
              storage = AnnotationStorage(migrate=True),
              #validators = ('isTidyHtmlWithCleanup',),
              #validators = ('isTidyHtml',),
              default_output_type = 'text/x-html',
              widget=RichWidget(
                        description = "",
                        description_msgid = "help_body_text",
                        label = "Body Text",
                        label_msgid = "label_body_text",
                        rows = 25,
                        i18n_domain = "plone"),
              ),

    LinesField('answerOptions',
        searchable=0,
        required=0,
        default=("Yes", "No"),
        widget=LinesWidget(
            label="Answer options",
            label_msgid="label_answer_options",
            description="""Enter the options you want to be available to the user here.
                           This only applies to the selection box, multiple selection box,
                           checkbox and radio button input types (selected below).
                           The text input options allow the user to type the input
                           rather than selecting from options. Press enter to seperate
                           the options.""",
            description_msgid="help_answer_options"),
        ),

    StringField('inputType',
        searchable=0,
        required=0,
        vocabulary=INPUT_TYPE,
        widget=SelectionWidget(
            label="Input Type",
            label_msgid="label_input_type",
            description="Please select what type of input you would like to use for this question.",
            description_msgid="help_input_type"),
        ),
    StringField('answers',
        searchable=0,
        required=0,
        default={},
        widget=StringWidget(visible=0,),
        ),

    StringField('Version',
        searchable=0,
        required=0,
        default='',
        widget=StringWidget(
                        visible=1,
                        label="File Version",
                        label_msgid="label_input_type",
                        description="Enter the file/software version number.",
                        description_msgid="help_input_type"
                        ),
        ),
StringField('ReleaseDate',
        searchable=0,
        required=0,
        default='',
        widget=CalendarWidget(
                        visible=1,
                        show_hm = False,
                        label="ReleaseDate",
                        label_msgid="label_input_type",
                        description="Enter the release date.",
                        description_msgid="help_input_type"
                        ),
        ),
    StringField('mirror1',
        searchable=0,
        required=0,
        default='',
        widget=StringWidget(
                        visible=1,
                        label="Mirror #1 URL",
                        label_msgid="label_input_type",
                        description="Please enter the full URL for the mirror. Include the http:// prefix.",
                        description_msgid="help_input_type"
                        ),
    ),
    StringField('mirror2',
        searchable=0,
        required=0,
        default='',
        widget=StringWidget(
                        visible=1,
                        label="Mirror #2 URL",
                        label_msgid="label_input_type",
                        description="Please enter the full URL for the mirror. Include the http:// prefix.",
                        description_msgid="help_input_type"
                        ),
    ),
    
    StringField('mirror3',
        searchable=0,
        required=0,
        default='',
        widget=StringWidget(
                        visible=1,
                        label="Mirror #3 URL",
                        label_msgid="label_input_type",
                        description="Please enter the full URL for the mirror. Include the http:// prefix.",
                        description_msgid="help_input_type"
                        ),
    ),
      ))

finalizeATCTSchema(schema)

class FileWithCopyright(ATCTContent):
    """A file with copyright"""
    archetype_name = portal_type = "FileWithCopyright"
    meta_type = "FileWithCopyright"
#    content_icon = "file_icon.gif"

    schema = schema
    actions = ATCTContent.actions
    #immediate_view = "base_edit"

    global_allow = 0
    filter_content_types = 1
    allowed_content_types = ()
    include_default_actions = 1
    _at_rename_after_creation = True

    security = ClassSecurityInfo()

    # Set up our views - these are available from the 'display' menu
    default_view = 'CopyrightedFile_view_FileWithCopyright'
    immediate_view = 'CopyrightedFile_view_FileWithCopyright'
    suppl_views = ('CopyrightedFile_view_FileWithCopyright','CopyrightedFile_view_MultipleFilesWithCopyright')

    # Set security for the object itslef

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'resetForUser')
    def resetForUser(self, userid):
        """Remove answer for a single user"""
        answers = self.getAnswers()
        if answers.has_key(userid):
            del answers[userid]
        self.setAnswers(answers)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'addAnswer')
    def addAnswer(self, userid, value, comments=""):
        """Add an answer and optional comments for a user."""
        answers = self.getAnswers()

        answers[userid] = {'value':value,
                           'comments':comments}
        self.setAnswers(answers)

    security.declareProtected(EDIT_CONTENTS_PERMISSION, 'getAnswerFor')
    def getAnswerFor(self, userid):
        """Get a specific user's answer"""
        answer = self.answers.get(userid, {}).get('value', None)
        if self.getInputType() in ['multipleSelect', 'checkbox']:
            if type(answer) == 'NoneType':
                return []
        return answer

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'download')
    def download(self, REQUEST, RESPONSE):
        """Download the file
        """
        licenseOK = licenseAgreed(self)
        if licenseOK == 1:
            field = self.getField('CopyrightedFile')
            return field.download(self)
            
        else:
            exit = self.getId() + '/view'
            return  self.REQUEST.RESPONSE.redirect(exit)

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'getIcon')
    def getIcon(self, relative_to_portal=0):
        """Calculate the icon using the mime type of the file
        """
        field = self.getField('CopyrightedFile')
        if not field or not self.get_size():
            # field is empty
            return BaseContent.getIcon(self, relative_to_portal)

        contenttype       = field.getContentType(self)
        contenttype_major = contenttype and contenttype.split('/')[0] or ''

        mtr   = getToolByName(self, 'mimetypes_registry', None)
        utool = getToolByName( self, 'portal_url' )

        if ICONMAP.has_key(contenttype):
            icon = quote(ICONMAP[contenttype])
        elif ICONMAP.has_key(contenttype_major):
            icon = quote(ICONMAP[contenttype_major])
        else:
            mimetypeitem = None
            try:
                mimetypeitem = mtr.lookup(contenttype)
            except MimeTypeException, msg:
                LOG.error('MimeTypeException for %s. Error is: %s' % (self.absolute_url(), str(msg)))
            if not mimetypeitem:
                return BaseContent.getIcon(self, relative_to_portal)
            icon = mimetypeitem[0].icon_path

        if relative_to_portal:
            return icon
        else:
            # Relative to REQUEST['BASEPATH1']
            res = utool(relative=1) + '/' + icon
            while res[:1] == '/':
                res = res[1:]
            return res

    security.declareProtected(VIEW_CONTENTS_PERMISSION, 'icon')
    def icon(self):
        """for ZMI
        """
        return self.getIcon()


Globals.InitializeClass(FileWithCopyright)
registerType(FileWithCopyright)